@@ -70,6 +70,22 @@ __imlib_BestVisual(Display * d, int screen, int *depth_return)
    return v;
 }
 
+static void
+_free_colors(Display * d, Colormap cmap, uint8_t * lut, int num)
+{
+   unsigned long       pixels[256];
+   int                 i;
+
+   if (num > 0)
+     {
+        for (i = 0; i < num; i++)
+           pixels[i] = (unsigned long)lut[i];
+        XFreeColors(d, cmap, pixels, num, 0);
+     }
+
+   free(lut);
+}
+
 static uint8_t     *
 __imlib_AllocColors332(Display * d, Colormap cmap, Visual * v)
 {
@@ -82,6 +98,8 @@ __imlib_AllocColors332(Display * d, Colormap cmap, Visual * v)
    sig_mask <<= (16 - v->bits_per_rgb);
    i = 0;
    color_lut = malloc(256 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
    for (r = 0; r < 8; r++)
      {
         for (g = 0; g < 8; g++)
@@ -101,21 +119,12 @@ __imlib_AllocColors332(Display * d, Colormap cmap, Visual * v)
                   xcl.blue = (unsigned short)((val << 8) | (val));
                   xcl_in = xcl;
                   ret = XAllocColor(d, cmap, &xcl);
-                  if ((ret == Success) ||
+                  if ((ret == 0) ||
                       ((xcl_in.red & sig_mask) != (xcl.red & sig_mask)) ||
                       ((xcl_in.green & sig_mask) != (xcl.green & sig_mask)) ||
                       ((xcl_in.blue & sig_mask) != (xcl.blue & sig_mask)))
                     {
-                       unsigned long       pixels[256];
-                       int                 j;
-
-                       if (i > 0)
-                         {
-                            for (j = 0; j < i; j++)
-                               pixels[j] = (unsigned long)color_lut[j];
-                            XFreeColors(d, cmap, pixels, i, 0);
-                         }
-                       free(color_lut);
+                       _free_colors(d, cmap, color_lut, i);
                        return NULL;
                     }
                   color_lut[i] = xcl.pixel;
@@ -138,6 +147,8 @@ __imlib_AllocColors666(Display * d, Colormap cmap, Visual * v)
    sig_mask <<= (16 - v->bits_per_rgb);
    i = 0;
    color_lut = malloc(256 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
    for (r = 0; r < 6; r++)
      {
         for (g = 0; g < 6; g++)
@@ -157,21 +168,12 @@ __imlib_AllocColors666(Display * d, Colormap cmap, Visual * v)
                   xcl.blue = (unsigned short)(val);
                   xcl_in = xcl;
                   ret = XAllocColor(d, cmap, &xcl);
-                  if ((ret == Success) ||
+                  if ((ret == 0) ||
                       ((xcl_in.red & sig_mask) != (xcl.red & sig_mask)) ||
                       ((xcl_in.green & sig_mask) != (xcl.green & sig_mask)) ||
                       ((xcl_in.blue & sig_mask) != (xcl.blue & sig_mask)))
                     {
-                       unsigned long       pixels[256];
-                       int                 j;
-
-                       if (i > 0)
-                         {
-                            for (j = 0; j < i; j++)
-                               pixels[j] = (unsigned long)color_lut[j];
-                            XFreeColors(d, cmap, pixels, i, 0);
-                         }
-                       free(color_lut);
+                       _free_colors(d, cmap, color_lut, i);
                        return NULL;
                     }
                   color_lut[i] = xcl.pixel;
@@ -194,6 +196,8 @@ __imlib_AllocColors232(Display * d, Colormap cmap, Visual * v)
    sig_mask <<= (16 - v->bits_per_rgb);
    i = 0;
    color_lut = malloc(128 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
    for (r = 0; r < 4; r++)
      {
         for (g = 0; g < 8; g++)
@@ -213,21 +217,12 @@ __imlib_AllocColors232(Display * d, Colormap cmap, Visual * v)
                   xcl.blue = (unsigned short)((val << 8) | (val));
                   xcl_in = xcl;
                   ret = XAllocColor(d, cmap, &xcl);
-                  if ((ret == Success) ||
+                  if ((ret == 0) ||
                       ((xcl_in.red & sig_mask) != (xcl.red & sig_mask)) ||
                       ((xcl_in.green & sig_mask) != (xcl.green & sig_mask)) ||
                       ((xcl_in.blue & sig_mask) != (xcl.blue & sig_mask)))
                     {
-                       unsigned long       pixels[256];
-                       int                 j;
-
-                       if (i > 0)
-                         {
-                            for (j = 0; j < i; j++)
-                               pixels[j] = (unsigned long)color_lut[j];
-                            XFreeColors(d, cmap, pixels, i, 0);
-                         }
-                       free(color_lut);
+                       _free_colors(d, cmap, color_lut, i);
                        return NULL;
                     }
                   color_lut[i] = xcl.pixel;
@@ -250,6 +245,8 @@ __imlib_AllocColors222(Display * d, Colormap cmap, Visual * v)
    sig_mask <<= (16 - v->bits_per_rgb);
    i = 0;
    color_lut = malloc(64 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
    for (r = 0; r < 4; r++)
      {
         for (g = 0; g < 4; g++)
@@ -269,21 +266,12 @@ __imlib_AllocColors222(Display * d, Colormap cmap, Visual * v)
                   xcl.blue = (unsigned short)((val << 8) | (val));
                   xcl_in = xcl;
                   ret = XAllocColor(d, cmap, &xcl);
-                  if ((ret == Success) ||
+                  if ((ret == 0) ||
                       ((xcl_in.red & sig_mask) != (xcl.red & sig_mask)) ||
                       ((xcl_in.green & sig_mask) != (xcl.green & sig_mask)) ||
                       ((xcl_in.blue & sig_mask) != (xcl.blue & sig_mask)))
                     {
-                       unsigned long       pixels[256];
-                       int                 j;
-
-                       if (i > 0)
-                         {
-                            for (j = 0; j < i; j++)
-                               pixels[j] = (unsigned long)color_lut[j];
-                            XFreeColors(d, cmap, pixels, i, 0);
-                         }
-                       free(color_lut);
+                       _free_colors(d, cmap, color_lut, i);
                        return NULL;
                     }
                   color_lut[i] = xcl.pixel;
@@ -306,6 +294,8 @@ __imlib_AllocColors221(Display * d, Colormap cmap, Visual * v)
    sig_mask <<= (16 - v->bits_per_rgb);
    i = 0;
    color_lut = malloc(32 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
    for (r = 0; r < 4; r++)
      {
         for (g = 0; g < 4; g++)
@@ -326,21 +316,12 @@ __imlib_AllocColors221(Display * d, Colormap cmap, Visual * v)
                   xcl.blue = (unsigned short)((val << 8) | (val));
                   xcl_in = xcl;
                   ret = XAllocColor(d, cmap, &xcl);
-                  if ((ret == Success) ||
+                  if ((ret == 0) ||
                       ((xcl_in.red & sig_mask) != (xcl.red & sig_mask)) ||
                       ((xcl_in.green & sig_mask) != (xcl.green & sig_mask)) ||
                       ((xcl_in.blue & sig_mask) != (xcl.blue & sig_mask)))
                     {
-                       unsigned long       pixels[256];
-                       int                 j;
-
-                       if (i > 0)
-                         {
-                            for (j = 0; j < i; j++)
-                               pixels[j] = (unsigned long)color_lut[j];
-                            XFreeColors(d, cmap, pixels, i, 0);
-                         }
-                       free(color_lut);
+                       _free_colors(d, cmap, color_lut, i);
                        return NULL;
                     }
                   color_lut[i] = xcl.pixel;
@@ -363,6 +344,8 @@ __imlib_AllocColors121(Display * d, Colormap cmap, Visual * v)
    sig_mask <<= (16 - v->bits_per_rgb);
    i = 0;
    color_lut = malloc(16 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
    for (r = 0; r < 2; r++)
      {
         for (g = 0; g < 4; g++)
@@ -384,21 +367,12 @@ __imlib_AllocColors121(Display * d, Colormap cmap, Visual * v)
                   xcl.blue = (unsigned short)((val << 8) | (val));
                   xcl_in = xcl;
                   ret = XAllocColor(d, cmap, &xcl);
-                  if ((ret == Success) ||
+                  if ((ret == 0) ||
                       ((xcl_in.red & sig_mask) != (xcl.red & sig_mask)) ||
                       ((xcl_in.green & sig_mask) != (xcl.green & sig_mask)) ||
                       ((xcl_in.blue & sig_mask) != (xcl.blue & sig_mask)))
                     {
-                       unsigned long       pixels[256];
-                       int                 j;
-
-                       if (i > 0)
-                         {
-                            for (j = 0; j < i; j++)
-                               pixels[j] = (unsigned long)color_lut[j];
-                            XFreeColors(d, cmap, pixels, i, 0);
-                         }
-                       free(color_lut);
+                       _free_colors(d, cmap, color_lut, i);
                        return NULL;
                     }
                   color_lut[i] = xcl.pixel;
@@ -421,6 +395,8 @@ __imlib_AllocColors111(Display * d, Colormap cmap, Visual * v)
    sig_mask <<= (16 - v->bits_per_rgb);
    i = 0;
    color_lut = malloc(8 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
    for (r = 0; r < 2; r++)
      {
         for (g = 0; g < 2; g++)
@@ -443,21 +419,12 @@ __imlib_AllocColors111(Display * d, Colormap cmap, Visual * v)
                   xcl.blue = (unsigned short)((val << 8) | (val));
                   xcl_in = xcl;
                   ret = XAllocColor(d, cmap, &xcl);
-                  if ((ret == Success) ||
+                  if ((ret == 0) ||
                       ((xcl_in.red & sig_mask) != (xcl.red & sig_mask)) ||
                       ((xcl_in.green & sig_mask) != (xcl.green & sig_mask)) ||
                       ((xcl_in.blue & sig_mask) != (xcl.blue & sig_mask)))
                     {
-                       unsigned long       pixels[256];
-                       int                 j;
-
-                       if (i > 0)
-                         {
-                            for (j = 0; j < i; j++)
-                               pixels[j] = (unsigned long)color_lut[j];
-                            XFreeColors(d, cmap, pixels, i, 0);
-                         }
-                       free(color_lut);
+                       _free_colors(d, cmap, color_lut, i);
                        return NULL;
                     }
                   color_lut[i] = xcl.pixel;
@@ -473,19 +440,32 @@ __imlib_AllocColors1(Display * d, Colormap cmap, Visual * v)
 {
    XColor              xcl;
    uint8_t            *color_lut;
+   int                 i;
 
    color_lut = malloc(2 * sizeof(uint8_t));
+   if (!color_lut)
+      return NULL;
+
+   i = 0;
    xcl.red = (unsigned short)(0x0000);
    xcl.green = (unsigned short)(0x0000);
    xcl.blue = (unsigned short)(0x0000);
-   XAllocColor(d, cmap, &xcl);
-   color_lut[0] = xcl.pixel;
+   if (!XAllocColor(d, cmap, &xcl))
+      goto bail;
+   color_lut[i++] = xcl.pixel;
+
    xcl.red = (unsigned short)(0xffff);
    xcl.green = (unsigned short)(0xffff);
    xcl.blue = (unsigned short)(0xffff);
-   XAllocColor(d, cmap, &xcl);
-   color_lut[1] = xcl.pixel;
+   if (!XAllocColor(d, cmap, &xcl))
+      goto bail;
+   color_lut[i] = xcl.pixel;
+
    return color_lut;
+
+ bail:
+   _free_colors(d, cmap, color_lut, i);
+   return NULL;
 }
 
 uint8_t            *
