@@ -9,7 +9,7 @@
 typedef struct {
    struct jpeg_error_mgr jem;
    sigjmp_buf          setjmp_buffer;
-   DATA8              *data;
+   uint8_t            *data;
 } ImLib_JPEG_data;
 
 static void
@@ -65,22 +65,27 @@ int
 load2(ImlibImage * im, int load_data)
 {
    int                 w, h, rc;
+   void               *fdata;
    struct jpeg_decompress_struct jds;
    ImLib_JPEG_data     jdata;
-   DATA8              *ptr, *line[16];
-   DATA32             *ptr2;
+   uint8_t            *ptr, *line[16];
+   uint32_t           *ptr2;
    int                 x, y, l, scans, inc;
    ExifInfo            ei = { 0 };
 
+   rc = LOAD_FAIL;
+
+   fdata = mmap(NULL, im->fsize, PROT_READ, MAP_SHARED, fileno(im->fp), 0);
+   if (fdata == MAP_FAILED)
+      return rc;
+
    /* set up error handling */
    jds.err = _jdata_init(&jdata);
    if (sigsetjmp(jdata.setjmp_buffer, 1))
       QUIT_WITH_RC(LOAD_FAIL);
 
-   rc = LOAD_FAIL;
-
    jpeg_create_decompress(&jds);
-   jpeg_stdio_src(&jds, im->fp);
+   jpeg_mem_src(&jds, fdata, im->fsize);
    jpeg_save_markers(&jds, JPEG_APP0 + 1, 256);
    jpeg_read_header(&jds, TRUE);
 
@@ -243,8 +248,7 @@ load2(ImlibImage * im, int load_data)
  quit:
    jpeg_destroy_decompress(&jds);
    free(jdata.data);
-   if (rc <= 0)
-      __imlib_FreeData(im);
+   munmap(fdata, im->fsize);
 
    return rc;
 }
@@ -256,15 +260,15 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    struct jpeg_compress_struct jcs;
    ImLib_JPEG_data     jdata;
    FILE               *f;
-   DATA8              *buf;
-   DATA32             *ptr;
+   uint8_t            *buf;
+   uint32_t           *ptr;
    JSAMPROW           *jbuf;
    int                 y, quality, compression;
    ImlibImageTag      *tag;
    int                 i, j;
 
    /* allocate a small buffer to convert image data */
-   buf = malloc(im->w * 3 * sizeof(DATA8));
+   buf = malloc(im->w * 3 * sizeof(uint8_t));
    if (!buf)
       return LOAD_FAIL;
 
@@ -331,7 +335,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
         /* convcert scaline from ARGB to RGB packed */
         for (j = 0, i = 0; i < im->w; i++)
           {
-             DATA32              pixel = *ptr++;
+             uint32_t            pixel = *ptr++;
 
              buf[j++] = PIXEL_R(pixel);
              buf[j++] = PIXEL_G(pixel);
